@@ -8,28 +8,31 @@ module Text.Blaze.Tests
 import Prelude hiding (div, id, null)
 import Data.Monoid (mempty, mappend)
 import Control.Monad (replicateM)
-import Control.Applicative ((<$>))
+import Control.Applicative (Applicative (..), (<$>))
 import Data.Word (Word8)
-import Data.Char (ord)
+import Data.Char (ord, isControl)
 import qualified Data.List as List
+import qualified Prelude as Prelude
 
-import Test.Framework (Test)
+import Test.Tasty (TestTree)
 import Test.HUnit (Assertion, (@=?))
-import Test.Framework.Providers.HUnit (testCase)
-import Test.Framework.Providers.QuickCheck2 (testProperty)
+import Test.Tasty.HUnit (testCase)
+import Test.Tasty.QuickCheck (testProperty)
 import Test.QuickCheck
 import qualified Data.ByteString as SB
 import qualified Data.ByteString.Lazy as LB
 import qualified Data.ByteString.Lazy.Char8 as LBC
 
+import Text.Blaze
 import Text.Blaze.Internal
 import Text.Blaze.Tests.Util
 
-tests :: [Test]
+tests :: [TestTree]
 tests = [ testProperty "left identity Monoid law"  monoidLeftIdentity
         , testProperty "right identity Monoid law" monoidRightIdentity
         , testProperty "associativity Monoid law"  monoidAssociativity
         , testProperty "mconcat Monoid law"        monoidConcat
+        , testProperty "identity Applicative law"  applicativeIdentity
         , testProperty "post escaping characters"  postEscapingCharacters
         , testProperty "valid UTF-8"               isValidUtf8
         , testProperty "external </ sequence"      externalEndSequence
@@ -40,6 +43,7 @@ tests = [ testProperty "left identity Monoid law"  monoidLeftIdentity
         , testCase     "contents 1"                contents1
         , testCase     "empty 1"                   empty1
         , testCase     "empty 2"                   empty2
+        , testCase     "comment 1"                 comment1
         ]
 
 -- | The left identity Monoid law.
@@ -62,6 +66,11 @@ monoidAssociativity x y z = (x >> (y >> z)) == ((x >> y) >> z)
 monoidConcat :: [Markup] -> Bool
 monoidConcat xs = sequence_ xs == foldr (>>) (return ()) xs
 
+-- | Applicative identity law.
+--
+applicativeIdentity :: Markup -> Bool
+applicativeIdentity x = (pure Prelude.id <*> x) == x
+
 -- | Escaped content cannot contain certain characters.
 --
 postEscapingCharacters :: String -> Bool
@@ -132,6 +141,7 @@ contents1 = "Hello World!" @=? renderUsingUtf8 (contents html)
     html :: Markup
     html = div $ do
         p ! id "para" $ "Hello "
+        stringComment "Test test"
         img ! name "An image"
         p "World!"
 
@@ -150,6 +160,15 @@ empty2 = False @=? null html
     html :: Markup
     html = "" `mappend` "" `mappend` p "a"
 
+comment1 :: Assertion
+comment1 = preEscapedString "<div>Hello <!-- Test --> World!</div>" @=? html
+  where
+    html :: Markup
+    html = div $ do
+        "Hello "
+        stringComment "Test"
+        " World!"
+
 -- Show instance for the HTML type, so we can debug.
 --
 instance Show Markup where
@@ -206,11 +225,14 @@ arbitraryMarkup depth = do
 
     -- Generate arbitrary string element.
     arbitraryString = do
-        s <- arbitrary
+        s <- genString
         return $ string s
 
     -- Generate an arbitrary HTML attribute.
     arbitraryAttribute = do
         attr <- elements [id, class_, name]
-        value <- arbitrary
+        value <- genString
         return $ attr $ stringValue value
+
+    -- Don't use control characters
+    genString = filter (not . isControl) <$> arbitrary
